/*
The copyright in this software is being made available under this MPEG Reference Software Copyright License. This software may be subject to other third party and contributor rights, including patent rights, and no such rights are granted under this license.

Copyright (c) 2016-2017, Joanneum Research, Mitsubishi Electric Research Labs, Peking University, Telecom Italia, University of Surrey, Visual Atoms

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
*	Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
*	Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
*	Neither the names of Joanneum Research, Mitsubishi Electric Research Labs, Peking University, Telecom Italia, University of Surrey, Visual Atoms nor the names of their contributors may be used to endorse or promote products derived from this software without specific prior written permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#pragma once

#include "FeatureList.h"

#include "DescriptorTimeMap.h"
#include "CdvaException.h"

#include <iostream>

namespace mpeg7cdva
{

class CoordList {
	

public:
	CoordList() {}
	CoordList(mpeg7cdvs::FeatureList& fl);

	virtual unsigned int getX(int i) = 0;
	virtual unsigned int getY(int i) = 0;

};

class WrappedCoordList : public CoordList {
protected:
	mpeg7cdvs::FeatureList& mfl;

	
	virtual unsigned int clampIdx(unsigned int i) {
		if (i >= mfl.nFeatures()) {
			return 0;
		}
		else return i; 
	}

public:
	WrappedCoordList(mpeg7cdvs::FeatureList& fl) : mfl(fl) {}

	virtual unsigned int  getX(int i) { assert(i < mfl.nFeatures()); return mfl.features[i].x; };
	virtual unsigned int  getY(int i) { assert(i < mfl.nFeatures()); return mfl.features[i].y; };
};

class LeanCoordList : public CoordList{
protected:
	unsigned int* xCoord;
	unsigned int* yCoord;
	int nFeatures;

public:
	LeanCoordList();

	LeanCoordList(int size);

	LeanCoordList(mpeg7cdvs::FeatureList& fl);

	LeanCoordList(const LeanCoordList& other);

	LeanCoordList & operator= (const LeanCoordList & other);

	virtual ~LeanCoordList();

	virtual unsigned int  getX(int i) { return xCoord[i]; };
	virtual unsigned int  getY(int i) { return yCoord[i]; };

	virtual void  setX(int i, unsigned int x) { xCoord[i]=x; };
	virtual void  setY(int i, unsigned int y) { yCoord[i]=y; };

	virtual int getLength() { return nFeatures;  }

	virtual mpeg7cdvs::FeatureList& getFeatureList();

	virtual void getFeatureList(mpeg7cdvs::FeatureList& fl);
};

class CompressedFeatureList : public mpeg7cdvs::CompressedFeatureList {

public:

	CompressedFeatureList();

	CompressedFeatureList(const CompressedFeatureList& a, bool relevantOnly=false);


	CompressedFeatureList(const mpeg7cdvs::FeatureList & a, bool relevantOnly);
	
	CompressedFeatureList & operator=(CompressedFeatureList a);

	CompressedFeatureList & operator=(mpeg7cdvs::CompressedFeatureList a);

	void toCDVScfl(mpeg7cdvs::CompressedFeatureList& cfl);

	void allocate(int nFeatures, int descLen);

	virtual int matchDescriptors_oneWay(mpeg7cdvs::PointPairs &pairs, const CompressedFeatureList & otherFeatureList, float ratioThreshold, 
		DescriptorTimeMap& thisDtm, DescriptorTimeMap& otherDtm, CoordList& thisFl, CoordList& otherFl, int thisIdx, int otherIdx, int** distanceMatrix) const;

	virtual int matchDescriptors_twoWay(mpeg7cdvs::PointPairs &pairs, const CompressedFeatureList &otherFeatureList, float ratioThreshold,
		DescriptorTimeMap& thisDtm, DescriptorTimeMap& otherDtm, CoordList& thisFl, CoordList& otherFl, int thisIdx, int otherIdx, int** distanceMatrix) const;

	void getFeatureList(mpeg7cdvs::FeatureList& fl);

	std::vector<int> relevances;
};


} 	// end of namespace