/*
The copyright in this software is being made available under this MPEG Reference Software Copyright License. This software may be subject to other third party and contributor rights, including patent rights, and no such rights are granted under this license.

Copyright (c) 2016-2017, Joanneum Research, Mitsubishi Electric Research Labs, Peking University, Telecom Italia, University of Surrey, Visual Atoms

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
*	Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
*	Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
*	Neither the names of Joanneum Research, Mitsubishi Electric Research Labs, Peking University, Telecom Italia, University of Surrey, Visual Atoms nor the names of their contributors may be used to endorse or promote products derived from this software without specific prior written permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#pragma once


#include <math.h>
#include <assert.h>

#include <vector>

namespace mpeg7cdva
{

class DescriptorTimeMap  {

private:
	int bufsz;

protected:
	char* m_buffer;
	int m_nDescr;  // number of local descriptors
	int m_nAbsDescr; // number of descriptors 
	int m_nFrames; // number of frames being described (incl medoid)

	int m_nbits; // number of bits needed by buffer

	std::vector< std::vector<unsigned short> > descIndex;

public:
	std::vector<int> ldOffsets;

	/* if the object is created using the default contructur, init must be called before using the object  */
	DescriptorTimeMap();

	DescriptorTimeMap(DescriptorTimeMap& other);

	DescriptorTimeMap(int nDescr, int nAbsDescr, int nFrames);

	/** construct from parsed buffer 
	  * contents of buffer will be copied into buffer under ownership of class
	  */
	DescriptorTimeMap(int nDescr, int nAbsDescr, int nFrames, char* buf);

	virtual ~DescriptorTimeMap();

	DescriptorTimeMap & operator= (const DescriptorTimeMap & other);

	void init(int nDescr, int nAbsDescr, int nFrames);

	void add(int descIdx, int frameIdx) { 
		int idx = (int)floor((descIdx*m_nFrames + frameIdx) * 4 / 8.0);

		int sh = (descIdx*m_nFrames + frameIdx) % 2 == 0 ? 0 : 4;

		m_buffer[idx] = m_buffer[idx] + (1 << sh);

		descIndex[frameIdx].push_back(descIdx);

	}

	bool get(int descIdx, int frameIdx) {
		return getCount(descIdx, frameIdx) > 0;
	}

	int getCount(int descIdx, int frameIdx) {
		int idx = (int)floor((descIdx*m_nFrames + frameIdx)*4 / 8.0);

		int sh = (descIdx*m_nFrames + frameIdx) % 2 == 0 ? 0 : 4;

		return ((m_buffer[idx] & (0xf << sh))  >> sh) ;
	}

	int getFeatureStartIndex(int descIdx, int frameIdx) {
		int cnt = 0;
		for (int i = 0; i < descIdx; i++) {
			cnt += getCount(i, frameIdx);
		}
		return cnt;
	}

	int getBitSize() const { return m_nbits; }

	char* getBuffer() const { return m_buffer;  }

	int getNDescr() const { return m_nDescr;  }

	int getNAbsDescr() const { return m_nAbsDescr;  }

	int getNFrames() const { return m_nFrames;  }

	std::vector<unsigned short>& getDescIndex(int frameIdx) { return descIndex[frameIdx]; }

	void addDescIndex(std::vector<unsigned short>& di, int frame);

	void writeDescIndex(unsigned char* buffer, bool compressed, int & outIdx) const;

	void parseDescIndex(unsigned char* buffer, std::vector<unsigned int>& nLocalDesc, bool compressed) ;

	void clear();

};


} 	// end of namespace
