/*
The copyright in this software is being made available under this MPEG Reference Software Copyright License. This software may be subject to other third party and contributor rights, including patent rights, and no such rights are granted under this license.

Copyright (c) 2016-2017, Joanneum Research, Mitsubishi Electric Research Labs, Peking University, Telecom Italia, University of Surrey, Visual Atoms

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
*	Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
*	Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
*	Neither the names of Joanneum Research, Mitsubishi Electric Research Labs, Peking University, Telecom Italia, University of Surrey, Visual Atoms nor the names of their contributors may be used to endorse or promote products derived from this software without specific prior written permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#pragma once

#include <stdio.h>
#include <string.h>

#include "Feature.h"

namespace mpeg7cdva {

	class RefFeature : public mpeg7cdvs::Feature {
	public:
		// overlapping with Feature

		int refFrame;
		int refFeatureIndex;
		int diff;
		int descrDiff[descrLength];
		char compressed[descrLength >> 2];
		bool refOnly;
		std::vector<unsigned int> timesPresent;


		RefFeature(mpeg7cdvs::Feature& other) {
			// copy members
			x = other.x;
			y = other.y;
			scale = other.scale;
			orientation = other.orientation;
			peak = other.peak;
			curvRatio = other.curvRatio;
			curvSigma = other.curvSigma;
			memcpy(&descr, &other.descr, 4 * descrLength);
			pdf = other.pdf;
			spatialIndex = other.spatialIndex;
			relevance = other.relevance;
			memcpy(&qdescr, &other.qdescr, 4 * descrLength);
			octave = other.octave;
			iscale = other.iscale;

			// init new members
			refFeatureIndex = -1;
			diff = 0;
			memset(&descrDiff, 0, 4 * descrLength);
			refOnly = false;

			compress();

		}

		void compress() {
			unsigned char uc;
			int nof_elements = descrLength;
			int byte_offset = 0;
			int descr_bytes = 0;
			for (int i = 0; i < nof_elements; i++)
			{
				if (byte_offset == 0)
				{
					uc = 0;
				}
				int val = qdescr[i];
				if (val == 2) val = 3;
				uc |= (val & 3) << byte_offset;
				byte_offset += 2;
				if (byte_offset == 8)
				{
					compressed[descr_bytes] = uc;
					byte_offset = 0;
					// go to next byte;
					descr_bytes++;
				}
			}
		}
	};


}